/*
    Klat4 - A LaTeX editor for KDE4

    Copyright (C) 2008  Anna A. Ladilova (ladilova@algebraic.ru)
    Copyright (C) 2002-2004  Jori Liesenborgs (jori@lumumba.luc.ac.be)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "klat.h"
#include "klatedittab.h"
#include "klattooltab.h"
#include "klatoutputtab.h"
#include "klatexecsettingsdlgimpl.h"
#include "klatsplashdialogimpl.h"

#include <KStandardAction>
#include <KActionCollection>
#include <KMenu>
#include <KApplication>
#include <KRecentFilesAction>
#include <KStatusBar>
#include <KMessageBox>
#include <KTextEditor/View>
#include <KConfigGroup>
#include <KLocalizedString>
#include <KLocale>
#include <QDockWidget>

#define ID_EXEC_SETTINGS_BASE						17000
#define ID_STATUS_CURSPOS						16000
#define ID_STATUS_LATEXCODE						16001

#define SECTCOMP	"compile"
#define SECTCONV	"convert"
#define SECTVIEW	"view"

Klat::Klat()
{
	config = new KConfig("klat4rc");
	execConfig.read(config);
	createWindows();
	readGlobalProperties(config);
	createMenuBarAndToolbarAndStatusBar();
	KGlobal::locale()->insertCatalog("katepart4");
	tooltab->connectToEditTab(edittab);
	tooltab->connectToKlatWidget(this);
	
	connect(edittab,SIGNAL(signalCursorPositionChanged(int,int)),this,SLOT(slotCursorPositionChanged(int,int)));
	connect(edittab,SIGNAL(signalFileOpened(const KUrl &)),this,SLOT(slotFileReferenced(const KUrl &)));
	connect(edittab,SIGNAL(signalFileSaved(const KUrl &)),this,SLOT(slotFileReferenced(const KUrl &)));
	connect(toolDock,SIGNAL(visibilityChanged(bool )),this,SLOT(slotViewCloseTools(bool )));
	connect(outputDock,SIGNAL(visibilityChanged(bool )),this,SLOT(slotViewCloseOutput(bool )));
	connect(edittab,SIGNAL(signalNewDocument(KTextEditor::View*)),this,SLOT(slotAddActions(KTextEditor::View*)));

	KlatSplashDialogImpl dlg(0,0);
	dlg.exec();
	show();
}

Klat::~Klat()
{
}

void Klat::createWindows()
{
	// Create the main dock widget

	edittab = new KlatEditTab(this);
	setCentralWidget(edittab);

	// Create the docking tool tab

	toolDock = new QDockWidget(this);
	toolDock->setObjectName("ToolDock");
	tooltab = new KlatToolTab(toolDock);
	toolDock->setWidget(tooltab);
	addDockWidget(Qt::LeftDockWidgetArea,toolDock);

	// Create the output tool tab

	outputDock = new QDockWidget(this);
	outputDock->setObjectName("OutputDock");
	outputtab = new KlatOutputTab(outputDock);
	outputDock->setWidget(outputtab);
	addDockWidget(Qt::BottomDockWidgetArea,outputDock);

	edittab->setOutputWidget(outputtab);
	edittab->setToolWidget(tooltab);
	resize(640,480);
}

void Klat::createMenuBarAndToolbarAndStatusBar()
{
	KAction *tmpAct;
	
	tmpAct = KStandardAction::openNew(edittab,SLOT(slotFileNew()),actionCollection());
	tmpAct = KStandardAction::open(edittab,SLOT(slotFileOpen()),actionCollection());

	recentFiles = KStandardAction::openRecent(edittab,SLOT(slotOpenUrl(const KUrl &)),actionCollection());
	KConfigGroup recentConfig(config,"Recent Files");
	recentFiles->loadEntries(recentConfig);
	
	tmpAct = KStandardAction::close(edittab,SLOT(slotFileClose()),actionCollection());
	tmpAct = KStandardAction::quit(KApplication::kApplication(),SLOT(closeAllWindows()),actionCollection());	
	connect(KApplication::kApplication(),SIGNAL(lastWindowClosed()),KApplication::kApplication(),SLOT(quit()));
	
	viewToolsAction = actionCollection()->addAction("show_tools",this,SLOT(slotViewToggleTools()));
	viewToolsAction->setText(i18n("Show &tools"));
	viewOutputAction = actionCollection()->addAction("show_output",this,SLOT(slotViewToggleOutput()));
	viewOutputAction->setText(i18n("Show &output"));

	execMenu = new KMenu();
	connect(execMenu,SIGNAL(triggered(QAction *)),this,SLOT(slotExecMenuItemClicked(QAction*)));

	tmpAct = actionCollection()->addAction("conf_exec_menu",this,SLOT(slotExecuteSettings()));
	tmpAct->setText(i18n("&Configure execute menu..."));

	tmpAct = actionCollection()->addAction("next_doc",edittab,SLOT(slotNextDocument()));
	tmpAct->setText(i18n("&Next document"));
	tmpAct = actionCollection()->addAction("prev_doc",edittab,SLOT(slotPrevDocument()));
	tmpAct->setText(i18n("&Previous document"));
	
	// set up the status bar
	statusBar()->insertItem("", ID_STATUS_LATEXCODE,80);
	statusBar()->setItemAlignment(ID_STATUS_LATEXCODE,Qt::AlignLeft|Qt::AlignVCenter);
	statusBar()->insertItem("", ID_STATUS_CURSPOS,20);
	statusBar()->setItemAlignment(ID_STATUS_CURSPOS,Qt::AlignRight|Qt::AlignVCenter);

	setupGUI(Default,"klat4ui.rc");
	buildExecMenu();
}

void Klat::slotExecuteSettings()
{
	KlatExecSettingsDlgImpl dlg(this);
	int status;

	dlg.setExecConfig(execConfig);
	status = dlg.exec();
	if (status != QDialog::Accepted)
		return;
	
	execConfig = dlg.getExecConfig();
	buildExecMenu();
}

void Klat::buildExecMenu()
{
	int i = 0;
	QList<QAction *> compileActions,convertActions,viewActions;
	QAction *tmpAct;
	
	if (!execConfig.gotoFirstEntry())
	{
		unplugActionList("ExecActionList_compile_merge");
		unplugActionList("ExecActionList_convert_merge");
		unplugActionList("ExecActionList_view_merge");
		return;
	}

	do
	{
		QString str = execConfig.menuName();
		QString sect = execConfig.section();

		if (str.length())
		{
			KShortcut cut = execConfig.shortcut();

			if (cut.isEmpty())
			{
				tmpAct=actionCollection()->addAction(str,this,SLOT(slotDummy()));
				tmpAct->setText(str);
				execMenu->addAction(tmpAct);
				if(sect == SECTCOMP)
					compileActions+=tmpAct;
				else if(sect == SECTCONV)
					convertActions+=tmpAct;
				else if(sect == SECTVIEW)
					viewActions+=tmpAct;
			}
			else
			{
				QList<QKeySequence> seq;
				QKeySequence kseq;

				seq=(QList<QKeySequence>)cut;
				kseq = seq.value(0);
				tmpAct=actionCollection()->addAction(str,this,SLOT(slotDummy()));
				tmpAct->setText(str);
				tmpAct->setShortcut(kseq);
				execMenu->addAction(tmpAct);
				if(sect == SECTCOMP)
					compileActions+=tmpAct;
				else if(sect ==  SECTCONV)
					convertActions+=tmpAct;
				else if(sect == SECTVIEW)
					viewActions+=tmpAct;
			}
		}
		i++;
	} while (execConfig.gotoNextEntry());
	
	unplugActionList("ExecActionList_compile_merge");
	plugActionList("ExecActionList_compile_merge",compileActions);
	unplugActionList("ExecActionList_convert_merge");
	plugActionList("ExecActionList_convert_merge",convertActions);
	unplugActionList("ExecActionList_view_merge");
	plugActionList("ExecActionList_view_merge",viewActions);
}	

void Klat::slotExecMenuItemClicked(QAction * action)
{
	if (!execConfig.gotoEntry(action->iconText()))
		return;
	edittab->executeCommand(execConfig.command(),execConfig.commandLine(),execConfig.saveFirst());
}

void Klat::slotDummy()
{
}

bool Klat::queryClose()
{
	KConfigGroup recentConfig(config,"Recent Files");

	if (!edittab->areAllDocumentsSaved())
	{
		int status = KMessageBox::warningYesNo(this,i18n("One or more documents are not saved. Do you still want to quit?"),i18n("File(s) not saved"));
		if (status != KMessageBox::Yes)
			return false;
	}
	recentFiles->saveEntries(recentConfig);
	saveGlobalProperties(config);
	execConfig.write(config);
	return true;
}

void Klat::slotCursorPositionChanged(int line,int col)
{
	if (line < 0 || col < 0)
		statusBar()->changeItem("",ID_STATUS_CURSPOS);
	else
	{
		QString str = i18n("Line: ") + QString::number(line+1) + i18n(" Col: ") + QString::number(col+1);
		statusBar()->changeItem(str,ID_STATUS_CURSPOS);
	}
}

void Klat::slotLaTeXCodeChanged(const QString &s)
{
	statusBar()->changeItem(s,ID_STATUS_LATEXCODE);
}

void Klat::slotLaTeXCodeClear()
{
	statusBar()->changeItem("",ID_STATUS_LATEXCODE);
}
void Klat::slotFileReferenced(const KUrl &url)
{
	recentFiles->addUrl(url);
}

void Klat::openCommandLineFile(const QString &fname)
{
	KUrl url(fname);

	if (url.isValid())
		edittab->slotOpenUrl(url);
}

void Klat::createFirstDocument()
{
	edittab->slotFileNew();
}

void Klat::slotViewToggleOutput()
{
	if (outputDock->isHidden())
		outputDock->setVisible(true);
	else
		outputDock->setVisible(false);
	slotUpdateViewMenu();
}

void Klat::slotViewToggleTools()
{
	if (toolDock->isHidden())
		toolDock->setVisible(true);
	else
		toolDock->setVisible(false);
	slotUpdateViewMenu();
}

void Klat::slotUpdateViewMenu()
{
	if (toolDock->isHidden())
		viewToolsAction->setText(i18n("Show &tools"));
	else
		viewToolsAction->setText(i18n("Hide &tools"));
	if (outputDock->isHidden())
		viewOutputAction->setText(i18n("Show &output"));
	else
		viewOutputAction->setText(i18n("Hide &output"));
}

void Klat::slotViewCloseTools(bool f)
{
	if(f == false)
		viewToolsAction->setText(i18n("Show &tools"));
	else
		viewToolsAction->setText(i18n("Hide &tools"));
	    
}

void Klat::slotViewCloseOutput(bool f)
{
	if(f == false)
		viewOutputAction->setText(i18n("Show &output"));
	else
		viewOutputAction->setText(i18n("Hide &output"));
}

void Klat::slotAddActions(KTextEditor::View *view)
{
	QList<QAction*> act;
    
	act+=view->actionCollection()->action("file_save");
	act+=view->actionCollection()->action("file_save_as");
	unplugActionList("FileActionList_save_merge");
	plugActionList("FileActionList_save_merge",act);
	act.clear();

	act+=view->actionCollection()->action("file_revert");
	unplugActionList("FileActionList_revert_merge");
	plugActionList("FileActionList_revert_merge",act);
	act.clear();

	act+=view->actionCollection()->action("file_print");
	unplugActionList("FileActionList_print_merge");
	plugActionList("FileActionList_print_merge",act);
	act.clear();

	act+=view->actionCollection()->action("edit_undo");
	act+=view->actionCollection()->action("edit_redo");
	unplugActionList("EditActionList_undo_redo_merge");
	plugActionList("EditActionList_undo_redo_merge",act);
	act.clear();

	act+=view->actionCollection()->action("edit_cut");
	act+=view->actionCollection()->action("edit_copy");
	act+=view->actionCollection()->action("edit_paste");
	unplugActionList("EditActionList_paste_merge");
	plugActionList("EditActionList_paste_merge",act);
	act.clear();

	act+=view->actionCollection()->action("edit_select_all");
	act+=view->actionCollection()->action("edit_deselect");
	act+=view->actionCollection()->action("set_verticalSelect");
	unplugActionList("EditActionList_select_merge");
	plugActionList("EditActionList_select_merge",act);
	act.clear();

	act+=view->actionCollection()->action("edit_find");
	act+=view->actionCollection()->action("edit_find_next");
	act+=view->actionCollection()->action("edit_find_prev");
	act+=view->actionCollection()->action("edit_replace");
	act+=view->actionCollection()->action("edit_find_selected");
	act+=view->actionCollection()->action("edit_find_selected_backwards");
	unplugActionList("EditActionList_find_merge");
	plugActionList("EditActionList_find_merge",act);

	act+=view->actionCollection()->action("go_goto_line");
	unplugActionList("EditActionList_go_merge");
	plugActionList("EditActionList_go_merge",act);
	act.clear();

	act+=view->actionCollection()->action("tools_toggle_write_lock");
	act+=view->actionCollection()->action("set_encoding");
	act+=view->actionCollection()->action("set_eol");
	unplugActionList("ToolsActionList_operations_merge");
	plugActionList("ToolsActionList_operations_merge",act);
	act.clear();

	act+=view->actionCollection()->action("tools_spelling");
	act+=view->actionCollection()->action("tools_spelling_from_cursor");
	unplugActionList("ToolsActionList_spelling_merge");
	plugActionList("ToolsActionList_spelling_merge",act);
	act.clear();
    
	act+=view->actionCollection()->action("tools_indent");
	act+=view->actionCollection()->action("tools_unindent");
	act+=view->actionCollection()->action("tools_cleanIndent");
	act+=view->actionCollection()->action("tools_align");
	unplugActionList("ToolsActionList_indent_merge");
	plugActionList("ToolsActionList_indent_merge",act);
	act.clear();
    
	act+=view->actionCollection()->action("tools_comment");
	act+=view->actionCollection()->action("tools_uncomment");
	unplugActionList("ToolsActionList_comment_merge");
	plugActionList("ToolsActionList_comment_merge",act);
	act.clear();
    
	act+=view->actionCollection()->action("tools_uppercase");
	act+=view->actionCollection()->action("tools_lowercase");
	act+=view->actionCollection()->action("tools_capitalize");
	unplugActionList("ToolsActionList_register_merge");
	plugActionList("ToolsActionList_register_merge",act);
	act.clear();
    
	act+=view->actionCollection()->action("tools_join_lines");
	act+=view->actionCollection()->action("tools_apply_wordwrap");
	unplugActionList("ToolsActionList_lines_merge");
	plugActionList("ToolsActionList_lines_merge",act);
	act.clear();
    
	act+=view->actionCollection()->action("set_confdlg");
	unplugActionList("SettingsActionList_config_merge");
	plugActionList("SettingsActionList_config_merge",act);
	act.clear();
    
	act+=view->actionCollection()->action("bookmarks");
	unplugActionList("BookmarksActionList");
	plugActionList("BookmarksActionList",act);
	act.clear();
}
